import os

from django.test import TestCase, override_settings

import gulp_rev


class gulp_revTestCase(TestCase):
    def setUp(self):
        self.path = 'css/main.css'
        self.manifest_path = 'rev-manifest.json'
        gulp_rev._STATIC_MAPPING = None

    def test_is_debug_false(self):
        self.assertFalse(gulp_rev.is_debug())

    @override_settings(DEBUG=True)
    def test_is_debug_true(self):
        self.assertTrue(gulp_rev.is_debug())

    def test_dev_url(self):
        dev_url = gulp_rev.dev_url(self.path)
        self.assertEqual(len(self.path)+9, len(dev_url))
        self.assertIn('?', dev_url)

    @override_settings(STATIC_ROOT=os.path.dirname(os.path.realpath(__file__)))
    def test_get_mapping_default(self):
        mapping = gulp_rev._get_mapping()
        self.assertIn(self.path, mapping)

    @override_settings(DJANGO_GULP_REV_PATH=os.path.join(os.path.dirname(os.path.realpath(__file__)), 'rev-manifest.json'))
    def test_get_mapping_with_setting(self):
        mapping = gulp_rev._get_mapping()
        self.assertIn(self.path, mapping)

    @override_settings(STATIC_ROOT=os.path.dirname(os.path.realpath(__file__)))
    def test_production_url(self):
        url = gulp_rev.production_url(self.path, '/static/'+self.path)
        mapping = gulp_rev._get_mapping()
        self.assertEqual(url, '/static/'+mapping[self.path])

    @override_settings(STATIC_ROOT=os.path.dirname(os.path.realpath(__file__)))
    def test_production_url_non_existing_path(self):
        path = 'img/a.png'
        url = gulp_rev.production_url(path, '/static/'+path)
        self.assertEqual(url, '/static/'+path)

    @override_settings(STATIC_URL='/teststatic/', DEBUG=True)
    def test_static_rev_debug_true(self):
        url = gulp_rev.static_rev(self.path)
        self.assertIn('/teststatic/'+self.path+'?', url)
        self.assertEqual(len('/teststatic/'+self.path+'?')+8, len(url))

    @override_settings(STATIC_ROOT=os.path.dirname(os.path.realpath(__file__)),
        STATIC_URL='/teststatic/', DEBUG=False)
    def test_static_rev_debug_false(self):
        url = gulp_rev.static_rev(self.path)
        mapping = gulp_rev._get_mapping()
        self.assertEqual('/teststatic/'+mapping[self.path], url)

    @override_settings(DJANGO_GULP_REV_PATH='notexist.json', STATIC_URL='/teststatic/',
        DEBUG=False)
    def test_mapping_file_doesnt_exist(self):
        url = gulp_rev.static_rev(self.path)
        self.assertIn('/teststatic/'+self.path+'?', url)
        self.assertEqual(len('/teststatic/'+self.path+'?')+8, len(url))


DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.db',
    }
}
USE_TZ = False
SITE_ID = 1
SECRET_KEY = '{{secret_key}}'

ROOT_URLCONF = 'test_app.urls'
STATIC_URL = '/static/'
STATIC_ROOT = ''

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'gulp_rev',
    'test_app',
)



import os
import json

from django.conf import settings
from django.templatetags.static import StaticNode
from django.utils.crypto import get_random_string


_STATIC_MAPPING = None


def is_debug():
    return getattr(settings, 'DEBUG', False)

def _get_mapping():
    """
    Finds and loads gulp's rev-manifest.json file.
    In default, this is looking for rev-manifest.json file places in your
    STATIC_ROOT. Can be overriden to a specific location by setting
    DJANGO_GULP_REV_PATH.
    """
    global _STATIC_MAPPING

    if _STATIC_MAPPING is None:
        manifest_path = getattr(settings,
            'DJANGO_GULP_REV_PATH',
            os.path.join(getattr(settings, 'STATIC_ROOT', ''), 'rev-manifest.json'))
        
        try:
            with open(manifest_path) as manifest_file:
                _STATIC_MAPPING = json.load(manifest_file)
        except IOError:
            return None

    return _STATIC_MAPPING

def dev_url(original):
    """
    For a dev envioronment (DEBUG=True), appends 8 random characters at the
    end of original path created by Django's {% static %} template tag.
    """
    return "{}?{}".format(original, get_random_string(length=8))

def production_url(path, original):
    """
    For a production environment (DEBUG=False), replaces original path
    created by Django's {% static %} template tag with relevant path from
    our mapping.
    """
    mapping = _get_mapping()
    if mapping:
        if path in mapping:
            return original.replace(path, mapping[path])
        return original
    else:
        return dev_url(original)

def static_rev(path):
    """
    Gets a joined path with the STATIC_URL setting, and applies revisioning
    depending on DEBUG setting.

    Usage::
        {% load rev %}
        {% static_rev "css/base.css" %}

    Example::
        {% static_rev "css/base.css" %}
        On DEBUG=True will return: /static/css/base.css?d9wdjs
        On DEBUG=False will return: /static/css/base-d9wdjs.css
    """
    static_path = StaticNode.handle_simple(path)

    if is_debug():
        return dev_url(static_path)

    return production_url(path, static_path)
